package com.lancoo.studentgrowth.campuslife.service;

import org.slf4j.LoggerFactory;
import java.util.Comparator;
import java.util.stream.Collector;
import java.util.function.Function;
import java.util.stream.Collectors;
import com.lancoo.studentgrowth.healthy.entity.StuHealthyInfo;
import java.util.Map;
import java.time.LocalDate;
import com.lancoo.studentgrowth.publishaward.entity.StuRewardsAndPunishmentsPunishment;
import com.lancoo.studentgrowth.publishaward.entity.StuRewardsAndPunishmentsRewardinfo;
import com.lancoo.studentgrowth.campuslife.entity.StudevCampuslifeConsumption;
import com.lancoo.studentgrowth.campuslife.entity.StudevCampuslifeNetworkUsage;
import com.lancoo.studentgrowth.common.model.external.baseplatform.StudentInfo;
import java.util.LinkedList;
import java.time.temporal.Temporal;
import java.time.Duration;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import org.springframework.util.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import java.util.Collections;
import com.lancoo.studentgrowth.common.model.response.pk.PkProject;
import java.util.List;
import java.time.LocalDateTime;
import com.lancoo.studentgrowth.common.api.YunBasicApiService;
import com.lancoo.studentgrowth.healthy.mapper.StuHealthyInfoMapper;
import com.lancoo.studentgrowth.publishaward.mapper.StuRewardsAndPunishmentsPunishmentMapper;
import com.lancoo.studentgrowth.publishaward.mapper.StuRewardsAndPunishmentsRewardinfoMapper;
import com.lancoo.studentgrowth.campuslife.mapper.StudevCampuslifeConsumptionMapper;
import com.lancoo.studentgrowth.campuslife.mapper.StudevCampuslifeNetworkUsageMapper;
import org.slf4j.Logger;
import org.springframework.stereotype.Service;

@Service
public class StuPkServiceImpl implements StuPkService
{
    private static final Logger log;
    private final StudevCampuslifeNetworkUsageMapper networkUsageMapper;
    private final StudevCampuslifeConsumptionMapper consumptionMapper;
    private final StuRewardsAndPunishmentsRewardinfoMapper rewardInfoMapper;
    private final StuRewardsAndPunishmentsPunishmentMapper punishmentMapper;
    private final StuHealthyInfoMapper healthyInfoMapper;
    private final YunBasicApiService basicApiService;
    
    public StuPkServiceImpl(final StudevCampuslifeNetworkUsageMapper networkUsageMapper, final StudevCampuslifeConsumptionMapper consumptionMapper, final StuRewardsAndPunishmentsRewardinfoMapper rewardInfoMapper, final StuRewardsAndPunishmentsPunishmentMapper punishmentMapper, final YunBasicApiService basicApiService, final StuHealthyInfoMapper healthyInfoMapper) {
        this.networkUsageMapper = networkUsageMapper;
        this.consumptionMapper = consumptionMapper;
        this.rewardInfoMapper = rewardInfoMapper;
        this.punishmentMapper = punishmentMapper;
        this.basicApiService = basicApiService;
        this.healthyInfoMapper = healthyInfoMapper;
    }
    
    public List<PkProject> getNetworkPkDate(final String schoolId, final String classId, final String gradeId, final LocalDateTime startDate, final LocalDateTime endDate) {
        final List<StudentInfo> studentInfoList = this.basicApiService.getStudentInfo(schoolId, gradeId, classId);
        if (studentInfoList.isEmpty()) {
            return Collections.emptyList();
        }
        final QueryWrapper<StudevCampuslifeNetworkUsage> queryWrapper = (QueryWrapper<StudevCampuslifeNetworkUsage>)new QueryWrapper();
        queryWrapper.eq((Object)"school_id", (Object)schoolId);
        queryWrapper.eq(StringUtils.hasText(classId), (Object)"class_id", (Object)classId);
        queryWrapper.eq(StringUtils.hasText(gradeId), (Object)"grade_id", (Object)gradeId);
        queryWrapper.ge(startDate != null, (Object)"start_time", (Object)startDate);
        queryWrapper.le(startDate != null, (Object)"start_time", (Object)startDate);
        final List<StudevCampuslifeNetworkUsage> networkUsages = this.networkUsageMapper.selectList((Wrapper)queryWrapper);
        final double networkAverage = networkUsages.stream().mapToDouble(networkUsage -> (double)Duration.between(networkUsage.getStartTime(), networkUsage.getEndTime()).toMillis()).average().orElse(0.0);
        final PkProject project = new PkProject();
        project.setProjectName("平均上网时长");
        project.setScore(Double.valueOf(networkAverage / studentInfoList.size() / 60000.0));
        project.setUnit("分");
        final List<PkProject> projects = new LinkedList<PkProject>();
        final PkProject times = new PkProject();
        times.setProjectName("平均上网次数");
        times.setScore(Double.valueOf(networkUsages.size() / (double)studentInfoList.size()));
        times.setUnit("次");
        projects.add(times);
        projects.add(project);
        return projects;
    }
    
    public List<PkProject> getConsumptionPkData(final String schoolId, final String classId, final String gradeId, final LocalDateTime startDate, final LocalDateTime endDate) {
        final List<StudentInfo> studentInfoList = this.basicApiService.getStudentInfo(schoolId, gradeId, classId);
        if (studentInfoList.isEmpty()) {
            return Collections.emptyList();
        }
        final QueryWrapper<StudevCampuslifeConsumption> queryWrapper = (QueryWrapper<StudevCampuslifeConsumption>)new QueryWrapper();
        queryWrapper.eq((Object)"school_id", (Object)schoolId);
        queryWrapper.eq(StringUtils.hasText(classId), (Object)"class_id", (Object)classId);
        queryWrapper.eq(StringUtils.hasText(gradeId), (Object)"grade_id", (Object)gradeId);
        queryWrapper.ge(startDate != null, (Object)"order_timestamp", (Object)startDate);
        queryWrapper.le(startDate != null, (Object)"order_timestamp", (Object)startDate);
        final List<StudevCampuslifeConsumption> consumptionList = this.consumptionMapper.selectList((Wrapper)queryWrapper);
        final double count = consumptionList.stream().mapToDouble(consumption -> consumption.getCount().doubleValue()).average().orElse(0.0);
        final PkProject project = new PkProject();
        project.setProjectName("平均消费总额");
        project.setScore(Double.valueOf(count / studentInfoList.size()));
        project.setUnit("元");
        final List<PkProject> projects = new LinkedList<PkProject>();
        final PkProject times = new PkProject();
        times.setProjectName("平均消费次数");
        times.setScore(Double.valueOf(consumptionList.size() / (double)studentInfoList.size()));
        times.setUnit("次");
        projects.add(times);
        projects.add(project);
        return projects;
    }
    
    public List<PkProject> getRewardPkData(final String schoolId, final String classId, final String gradeId, final LocalDateTime startDate, final LocalDateTime endDate) {
        final List<StudentInfo> studentInfoList = this.basicApiService.getStudentInfo(schoolId, gradeId, classId);
        if (studentInfoList.isEmpty()) {
            return Collections.emptyList();
        }
        final QueryWrapper<StuRewardsAndPunishmentsRewardinfo> queryWrapper = (QueryWrapper<StuRewardsAndPunishmentsRewardinfo>)new QueryWrapper();
        queryWrapper.eq((Object)"school_id", (Object)schoolId);
        queryWrapper.eq(StringUtils.hasText(classId), (Object)"class_id", (Object)classId);
        queryWrapper.eq(StringUtils.hasText(gradeId), (Object)"grade_id", (Object)gradeId);
        queryWrapper.ge(startDate != null, (Object)"award_date", (Object)startDate);
        queryWrapper.le(endDate != null, (Object)"award_date", (Object)endDate);
        final List<StuRewardsAndPunishmentsRewardinfo> rewardinfoList = this.rewardInfoMapper.selectList((Wrapper)queryWrapper);
        final List<PkProject> projects = new LinkedList<PkProject>();
        final PkProject totalTimes = new PkProject();
        totalTimes.setProjectName("总获奖次数");
        totalTimes.setScore(Double.valueOf((double)rewardinfoList.size()));
        totalTimes.setUnit("次");
        final PkProject times = new PkProject();
        times.setProjectName("平均获奖次数");
        times.setScore(Double.valueOf(rewardinfoList.size() / (double)studentInfoList.size()));
        times.setUnit("次");
        projects.add(totalTimes);
        projects.add(times);
        return projects;
    }
    
    public List<PkProject> getPunishPkData(final String schoolId, final String classId, final String gradeId, final LocalDateTime startDate, final LocalDateTime endDate) {
        final List<StudentInfo> studentInfoList = this.basicApiService.getStudentInfo(schoolId, gradeId, classId);
        if (studentInfoList.isEmpty()) {
            return Collections.emptyList();
        }
        final QueryWrapper<StuRewardsAndPunishmentsPunishment> queryWrapper = (QueryWrapper<StuRewardsAndPunishmentsPunishment>)new QueryWrapper();
        queryWrapper.eq((Object)"school_id", (Object)schoolId);
        queryWrapper.eq(StringUtils.hasText(classId), (Object)"class_id", (Object)classId);
        queryWrapper.eq(StringUtils.hasText(gradeId), (Object)"grade_id", (Object)gradeId);
        queryWrapper.ge(startDate != null, (Object)"punishment_date", (Object)startDate);
        queryWrapper.le(endDate != null, (Object)"punishment_date", (Object)endDate);
        final List<StuRewardsAndPunishmentsPunishment> punishmentList = this.punishmentMapper.selectList((Wrapper)queryWrapper);
        final List<PkProject> projects = new LinkedList<PkProject>();
        final PkProject times = new PkProject();
        times.setProjectName("总获惩次数");
        times.setScore(Double.valueOf((double)punishmentList.size()));
        times.setUnit("次");
        final PkProject averageTimes = new PkProject();
        averageTimes.setProjectName("平均获惩次数");
        averageTimes.setScore(Double.valueOf(punishmentList.size() / (double)studentInfoList.size()));
        averageTimes.setUnit("次");
        projects.add(times);
        projects.add(averageTimes);
        return projects;
    }
    
    public List<PkProject> getHealthyPkData(final String schoolId, final String classId, final String gradeId, final LocalDate startDate, final LocalDate endDate) {
        final List<StudentInfo> studentInfoList = this.basicApiService.getStudentInfo(schoolId, gradeId, classId);
        if (studentInfoList.isEmpty()) {
            return Collections.emptyList();
        }
        final QueryWrapper<StuHealthyInfo> queryWrapper = (QueryWrapper<StuHealthyInfo>)new QueryWrapper();
        queryWrapper.eq((Object)"school_id", (Object)schoolId);
        queryWrapper.eq(StringUtils.hasText(classId), (Object)"class_id", (Object)classId);
        queryWrapper.eq(StringUtils.hasText(gradeId), (Object)"grade_id", (Object)gradeId);
        queryWrapper.gt(startDate != null, (Object)"record_time", (Object)startDate);
        queryWrapper.lt(endDate != null, (Object)"record_time", (Object)endDate);
        final List<StuHealthyInfo> healthyInfoList = this.healthyInfoMapper.selectList((Wrapper)queryWrapper);
        final Map<String, List<StuHealthyInfo>> stringListMap = healthyInfoList.stream().collect((Collector<? super Object, ?, Map<String, List<StuHealthyInfo>>>)Collectors.groupingBy((Function<? super Object, ? extends String>)StuHealthyInfo::getStudentId));
        final List<StuHealthyInfo> latestHealthyInfoList = stringListMap.values().stream().map(stuHealthyInfos -> stuHealthyInfos.stream().max(Comparator.comparing((Function<? super T, ? extends Comparable>)StuHealthyInfo::getRecordTime)).get()).collect((Collector<? super Object, ?, List<StuHealthyInfo>>)Collectors.toList());
        final List<PkProject> projects = new LinkedList<PkProject>();
        final PkProject averageHeight = new PkProject();
        averageHeight.setProjectName("平均身高");
        averageHeight.setScore(Double.valueOf(latestHealthyInfoList.stream().mapToDouble(StuHealthyInfo::getHeight).average().orElse(0.0)));
        averageHeight.setUnit("cm");
        final PkProject averageWeight = new PkProject();
        averageWeight.setProjectName("平均体重");
        averageWeight.setScore(Double.valueOf(latestHealthyInfoList.stream().mapToDouble(StuHealthyInfo::getWeight).average().orElse(0.0)));
        averageWeight.setUnit("kg");
        final PkProject averageBMI = new PkProject();
        averageBMI.setProjectName("平均BMI");
        averageBMI.setScore(Double.valueOf(latestHealthyInfoList.stream().mapToDouble(healthyInfo -> {
            final Float height = healthyInfo.getHeight();
            final Float weight = healthyInfo.getWeight();
            if (height == null || height == 0.0f || weight == null) {
                return 0.0;
            }
            else {
                return (double)(weight / (height / 100.0f * 2.0f));
            }
        }).average().orElse(0.0)));
        averageBMI.setUnit("");
        projects.add(averageHeight);
        projects.add(averageWeight);
        projects.add(averageBMI);
        return projects;
    }
    
    static {
        log = LoggerFactory.getLogger(StuPkServiceImpl.class);
    }
}
